Module: ReactOnRails::Helper
- Includes:
- FontHelper, ProHelper, Utils::Required
- Included in:
- ReactOnRailsHelper
- Defined in:
- lib/react_on_rails/helper.rb,
sig/react_on_rails/helper.rbs
Constant Summary collapse
- COMPONENT_HTML_KEY =
"componentHtml"- IMMEDIATE_HYDRATION_WARNING_MUTEX =
Mutex.new
Constants included from FontHelper
Class Method Summary collapse
-
.reset_removed_immediate_hydration_warnings! ⇒ void
:nodoc:.
-
.validate_redux_store_options!(rest) ⇒ Object
Validates the **rest kwargs accepted by both redux_store implementations (controller concern and view helper).
- .warn_removed_immediate_hydration_option(helper_name) ⇒ void
Instance Method Summary collapse
- #add_csp_nonce_to_context(result) ⇒ Object
-
#add_rsc_stream_observability_to_context(context) ⇒ Object
Set by ReactOnRailsPro::Stream#initialize_rsc_stream_observability_state.
-
#add_rsc_stream_observability_to_render_options(render_options) ⇒ Object
Set by ReactOnRailsPro::Stream#initialize_rsc_stream_observability_state.
- #build_react_component_result_for_server_rendered_hash(server_rendered_html: required("server_rendered_html"), component_specification_tag: required("component_specification_tag"), console_script: required("console_script"), render_options: required("render_options")) ⇒ Object
- #build_react_component_result_for_server_rendered_string(server_rendered_html: required("server_rendered_html"), component_specification_tag: required("component_specification_tag"), console_script: required("console_script"), render_options: required("render_options")) ⇒ Object
- #client_prop_target_key(props, key) ⇒ Object
- #compose_react_component_html_with_spec_and_console(component_specification_tag, rendered_output, console_script) ⇒ Object
- #create_render_options(react_component_name, options) ⇒ Object
-
#csp_nonce ⇒ Object
Returns the CSP script nonce for the current request, or nil if CSP is not enabled.
- #current_shakapacker_instance ⇒ Object
- #generated_component_pack_name(component_name) ⇒ Object
- #generated_components_pack_path(component_name) ⇒ Object
- #generated_stores_pack_path(store_name) ⇒ Object
- #in_mailer? ⇒ Boolean
- #initialize_redux_stores(render_options) ⇒ Object
- #internal_react_component(react_component_name, options = {}) ⇒ Object
- #json_safe_and_pretty(hash_or_string) ⇒ Object
- #load_pack_for_generated_component(_react_component_name, render_options) ⇒ Object
- #load_pack_for_generated_store(store_name, explicit_auto_load: false) ⇒ Object
- #merge_client_props(existing_props, client_props) ⇒ Object
- #merge_server_rendered_client_props!(render_options, result) ⇒ Object
- #modulepreload_source?(source) ⇒ Boolean
-
#normalize_js_stack_lines(stack) ⇒ Object
V8 stack strings begin with a
"TypeError: ..."header line — and chained-exception stacks add further non-frame headers mid-stack (e.g.Caused by: ...) — none of which match Ruby'sfile:line:in 'method'backtrace format. - #preload_crossorigin ⇒ Object
- #preload_link_attributes(source, href:) ⇒ Object
- #preload_link_for_javascript_source(source, href:) ⇒ Object
- #preload_link_for_source(source) ⇒ Object
- #preload_link_for_stylesheet_source(source, href:) ⇒ Object
- #preload_manifest_key?(source, key) ⇒ Boolean
- #preload_manifest_source(source) ⇒ Object
- #preload_manifest_value(source, key) ⇒ Object
- #preload_source_integrity(source) ⇒ Object
- #preload_source_path(source) ⇒ Object
- #preload_sources_for_generated_pack(pack_name) ⇒ Object
- #preload_sources_for_javascript_pack(pack_name) ⇒ Object
- #preload_sources_for_pack(pack_name, type:, required:) ⇒ Object
- #preload_sources_for_stylesheet_pack(pack_name) ⇒ Object
-
#prepend_render_rails_context(render_value) ⇒ safe_buffer
prepend the rails_context if not yet applied.
- #props_string(props) ⇒ Object
-
#rails_context(server_side: true) ⇒ Object
This is the definitive list of the default values used for the rails_context, which is the second parameter passed to both component and store Render-Functions.
- #rails_context_if_not_already_rendered ⇒ Object
- #raise_if_duplicate_client_prop_key_types!(props, key) ⇒ Object
- #raise_missing_autoloaded_bundle(react_component_name) ⇒ Object
- #raise_missing_autoloaded_store_bundle(store_name) ⇒ Object
- #raise_prerender_error(json_result, react_component_name, props, js_code) ⇒ Object
- #raise_renderer_prerender_error(err, react_component_name, props, js_code) ⇒ Object
-
#react_component(component_name, options = {}) ⇒ Object
Main helper methods Returns html_safe string (ActiveSupport::SafeBuffer).
-
#react_component_hash(component_name, options = {}) ⇒ Object
Returns hash with html_safe strings (ActiveSupport::SafeBuffer).
-
#react_on_rails_attribution_comment ⇒ Object
Generates the HTML attribution comment Pro version calls ReactOnRailsPro::Utils for license-specific details.
-
#react_on_rails_preload_links(*component_names) ⇒ safe_buffer
Returns html_safe string (ActiveSupport::SafeBuffer).
-
#redux_store(store_name, props: {}, defer: false, auto_load_bundle: nil, **rest) ⇒ Object
Returns html_safe string (ActiveSupport::SafeBuffer).
-
#redux_store_hydration_data ⇒ safe_buffer
Returns html_safe string (ActiveSupport::SafeBuffer).
-
#registered_stores ⇒ Object
rubocop:enable Metrics/AbcSize, Metrics/CyclomaticComplexity.
- #registered_stores_defer_render ⇒ Object
- #registered_stores_including_deferred ⇒ Object
- #render_redux_store_data(redux_store_data) ⇒ Object
- #rendering_error_from_result(json_result) ⇒ Object
- #sanitized_props_string(props) ⇒ String
-
#server_render_js(js_expression, options = {}) ⇒ Object
Returns html_safe string (ActiveSupport::SafeBuffer).
-
#server_rendered_react_component(render_options) ⇒ Object
Returns object with values that are NOT html_safe!.
- #shakapacker_integrity_config ⇒ Object
- #should_raise_streaming_prerender_error?(chunk_json_result, render_options) ⇒ Boolean
- #unique_preload_sources_by_href(sources) ⇒ Object
-
#wrap_console_script_with_nonce(console_script_code) ⇒ Object
Wraps console replay JavaScript code in a script tag with CSP nonce if available.
Methods included from FontHelper
ensure_safe!, fallback_font_face_rule, font_face_markup, font_face_rule, preload_link, #react_on_rails_font_face
Methods included from ProHelper
#generate_component_script, #generate_store_script, #generated_stylesheet_hrefs_json, #hydrate_on_data_attribute_value
Methods included from Utils::Required
Class Method Details
.reset_removed_immediate_hydration_warnings! ⇒ void
This method returns an undefined value.
:nodoc:
45 46 47 48 49 |
# File 'lib/react_on_rails/helper.rb', line 45 def reset_removed_immediate_hydration_warnings! IMMEDIATE_HYDRATION_WARNING_MUTEX.synchronize do @removed_immediate_hydration_warnings = {} end end |
.validate_redux_store_options!(rest) ⇒ Object
Validates the **rest kwargs accepted by both redux_store implementations (controller concern and view helper). Raises ArgumentError for unknown keywords and warns once if the removed :immediate_hydration option is passed.
54 55 56 57 58 59 60 61 62 63 |
# File 'lib/react_on_rails/helper.rb', line 54 def (rest) unknown_keys = rest.keys - [:immediate_hydration] if unknown_keys.any? plural = unknown_keys.one? ? "" : "s" = unknown_keys.map { |key| ":#{key}" }.join(", ") raise ArgumentError, "unknown keyword#{plural}: #{}" end warn_removed_immediate_hydration_option("redux_store") if rest.key?(:immediate_hydration) end |
.warn_removed_immediate_hydration_option(helper_name) ⇒ void
This method returns an undefined value.
29 30 31 32 33 34 35 36 37 38 39 40 41 42 |
# File 'lib/react_on_rails/helper.rb', line 29 def warn_removed_immediate_hydration_option(helper_name) should_warn = IMMEDIATE_HYDRATION_WARNING_MUTEX.synchronize do @removed_immediate_hydration_warnings ||= {} next false if @removed_immediate_hydration_warnings[helper_name] @removed_immediate_hydration_warnings[helper_name] = true end return unless should_warn Rails.logger.warn( "[React on Rails] `immediate_hydration:` is no longer supported on #{helper_name}. Remove this option." ) end |
Instance Method Details
#add_csp_nonce_to_context(result) ⇒ Object
434 435 436 437 |
# File 'lib/react_on_rails/helper.rb', line 434 def add_csp_nonce_to_context(result) nonce = csp_nonce result[:cspNonce] = nonce if nonce.present? end |
#add_rsc_stream_observability_to_context(context) ⇒ Object
Set by ReactOnRailsPro::Stream#initialize_rsc_stream_observability_state. Keep this ivar name in sync with the Pro stream concern.
417 418 419 420 421 422 |
# File 'lib/react_on_rails/helper.rb', line 417 def add_rsc_stream_observability_to_context(context) return unless defined?(@react_on_rails_rsc_stream_observability) && @react_on_rails_rsc_stream_observability context[:rscStreamObservability] = true end |
#add_rsc_stream_observability_to_render_options(render_options) ⇒ Object
Set by ReactOnRailsPro::Stream#initialize_rsc_stream_observability_state. Keep this ivar name in sync with the Pro stream concern.
426 427 428 429 430 431 432 |
# File 'lib/react_on_rails/helper.rb', line 426 def () return unless .streaming? return unless defined?(@react_on_rails_rsc_stream_observability) && @react_on_rails_rsc_stream_observability .set_option(:rsc_stream_observability, true) end |
#build_react_component_result_for_server_rendered_hash(server_rendered_html: required("server_rendered_html"), component_specification_tag: required("component_specification_tag"), console_script: required("console_script"), render_options: required("render_options")) ⇒ Object
691 692 693 694 695 696 697 698 699 700 701 702 703 704 705 706 707 708 709 710 711 712 713 714 715 716 717 718 719 720 721 722 723 |
# File 'lib/react_on_rails/helper.rb', line 691 def build_react_component_result_for_server_rendered_hash( server_rendered_html: required("server_rendered_html"), component_specification_tag: required("component_specification_tag"), console_script: required("console_script"), render_options: required("render_options") ) = . [:id] = .dom_id unless server_rendered_html[COMPONENT_HTML_KEY] raise ReactOnRails::Error, "server_rendered_html hash expected to contain \"#{COMPONENT_HTML_KEY}\" key." end rendered_output = content_tag(:div, server_rendered_html[COMPONENT_HTML_KEY].html_safe, ) result_console_script = .replay_console ? wrap_console_script_with_nonce(console_script) : "" result = compose_react_component_html_with_spec_and_console( component_specification_tag, rendered_output, result_console_script ) # Other HTML strings need to be marked as html_safe too: server_rendered_hash_except_component = server_rendered_html.except(COMPONENT_HTML_KEY) server_rendered_hash_except_component.each do |key, html_string| server_rendered_hash_except_component[key] = html_string.html_safe end result_with_rails_context = prepend_render_rails_context(result) { COMPONENT_HTML_KEY => result_with_rails_context }.merge( server_rendered_hash_except_component ) end |
#build_react_component_result_for_server_rendered_string(server_rendered_html: required("server_rendered_html"), component_specification_tag: required("component_specification_tag"), console_script: required("console_script"), render_options: required("render_options")) ⇒ Object
664 665 666 667 668 669 670 671 672 673 674 675 676 677 678 679 680 681 682 683 684 685 686 687 688 689 |
# File 'lib/react_on_rails/helper.rb', line 664 def build_react_component_result_for_server_rendered_string( server_rendered_html: required("server_rendered_html"), component_specification_tag: required("component_specification_tag"), console_script: required("console_script"), render_options: required("render_options") ) = . if .key?(:tag) = [:tag] .delete(:tag) else = "div" end [:id] = .dom_id rendered_output = content_tag(.to_sym, server_rendered_html.html_safe, ) result_console_script = .replay_console ? wrap_console_script_with_nonce(console_script) : "" result = compose_react_component_html_with_spec_and_console( component_specification_tag, rendered_output, result_console_script ) prepend_render_rails_context(result) end |
#client_prop_target_key(props, key) ⇒ Object
880 881 882 883 884 885 886 |
# File 'lib/react_on_rails/helper.rb', line 880 def client_prop_target_key(props, key) string_key = key.to_s symbol_key = string_key.to_sym # Preserve an existing symbol key when clientProps arrives from JSON with string keys, # otherwise we would create duplicate entries that serialize to the same JSON key. props.key?(symbol_key) && !props.key?(string_key) ? symbol_key : string_key end |
#compose_react_component_html_with_spec_and_console(component_specification_tag, rendered_output, console_script) ⇒ Object
755 756 757 758 759 760 761 762 |
# File 'lib/react_on_rails/helper.rb', line 755 def compose_react_component_html_with_spec_and_console(component_specification_tag, rendered_output, console_script) # IMPORTANT: Ensure that we mark string as html_safe to avoid escaping. added_html = "#{component_specification_tag}\n#{console_script}".strip added_html = added_html.present? ? "\n#{added_html}" : "" "#{rendered_output}#{added_html}".html_safe end |
#create_render_options(react_component_name, options) ⇒ Object
500 501 502 503 504 505 506 507 508 509 510 511 |
# File 'lib/react_on_rails/helper.rb', line 500 def (react_component_name, ) = .dup # If no store dependencies are passed, default to all registered stores up till now unless .key?(:store_dependencies) store_dependencies = registered_stores_including_deferred.map { |store| store[:store_name] } = .merge(store_dependencies: store_dependencies.presence) end = ReactOnRails::ReactComponent::RenderOptions.new(react_component_name:, options:) () end |
#csp_nonce ⇒ Object
Returns the CSP script nonce for the current request, or nil if CSP is not enabled. Rails 5.2-6.0 use content_security_policy_nonce with no arguments. Rails 6.1+ accept an optional directive argument.
728 729 730 731 732 733 734 735 736 737 |
# File 'lib/react_on_rails/helper.rb', line 728 def csp_nonce return unless respond_to?(:content_security_policy_nonce) begin content_security_policy_nonce(:script) rescue ArgumentError # Fallback for Rails versions that don't accept arguments content_security_policy_nonce end end |
#current_shakapacker_instance ⇒ Object
560 561 562 |
# File 'lib/react_on_rails/helper.rb', line 560 def current_shakapacker_instance ::Shakapacker.instance end |
#generated_component_pack_name(component_name) ⇒ Object
521 522 523 524 525 526 527 528 529 530 531 532 |
# File 'lib/react_on_rails/helper.rb', line 521 def generated_component_pack_name(component_name) component_name = component_name.to_s component_name = component_name.delete_prefix("generated/") unless component_name.match?(/\A[A-Za-z0-9_]+\z/) raise ArgumentError, "react_on_rails_preload_links component names must use PascalCase, camelCase, or snake_case " \ "without hyphens: #{component_name.inspect}" end "generated/#{component_name.camelize}" end |
#generated_components_pack_path(component_name) ⇒ Object
513 514 515 |
# File 'lib/react_on_rails/helper.rb', line 513 def generated_components_pack_path(component_name) "#{ReactOnRails::PackerUtils.packer_source_entry_path}/generated/#{component_name}.js" end |
#generated_stores_pack_path(store_name) ⇒ Object
517 518 519 |
# File 'lib/react_on_rails/helper.rb', line 517 def generated_stores_pack_path(store_name) "#{ReactOnRails::PackerUtils.packer_source_entry_path}/generated/#{store_name}.js" end |
#in_mailer? ⇒ Boolean
1054 1055 1056 1057 1058 1059 |
# File 'lib/react_on_rails/helper.rb', line 1054 def in_mailer? return false unless defined?(controller) return false unless defined?(ActionMailer::Base) controller.is_a?(ActionMailer::Base) end |
#initialize_redux_stores(render_options) ⇒ Object
1028 1029 1030 1031 1032 1033 1034 1035 1036 1037 1038 1039 1040 1041 1042 1043 1044 1045 1046 1047 1048 1049 1050 1051 1052 |
# File 'lib/react_on_rails/helper.rb', line 1028 def initialize_redux_stores() result = +<<-JS ReactOnRails.clearHydratedStores(); JS store_dependencies = .store_dependencies return result unless store_dependencies.present? declarations = +"var reduxProps, store, storeGenerator;\n" store_objects = registered_stores_including_deferred.select do |store| store_dependencies.include?(store[:store_name]) end result << store_objects.each_with_object(declarations) do |redux_store_data, memo| store_name = redux_store_data[:store_name] props = props_string(redux_store_data[:props]) memo << <<~JS reduxProps = #{props}; storeGenerator = ReactOnRails.getStoreGenerator(#{store_name.to_json}); store = storeGenerator(reduxProps, railsContext); ReactOnRails.setStore(#{store_name.to_json}, store); JS end result end |
#internal_react_component(react_component_name, options = {}) ⇒ Object
797 798 799 800 801 802 803 804 805 806 807 808 809 810 811 812 813 814 815 816 817 818 819 820 821 822 823 824 |
# File 'lib/react_on_rails/helper.rb', line 797 def internal_react_component(react_component_name, = {}) # Create the JavaScript and HTML to allow either client or server rendering of the # react_component. # # Create the JavaScript setup of the global to initialize the client rendering # (re-hydrate the data). This enables react rendered on the client to see that the # server has already rendered the HTML. = (react_component_name, ) load_pack_for_generated_component(react_component_name, ) # Create the HTML rendering part result = server_rendered_react_component() # clientProps are only expected on successful SSR hashes. Current error hashes do not # include that key, so non-SSR/error paths skip this merge entirely. merge_server_rendered_client_props!(, result) if result.is_a?(Hash) # Setup the page_loaded_js, which is the same regardless of prerendering or not! # The reason is that React is smart about not doing extra work if the server rendering did its job. component_specification_tag = generate_component_script() { render_options:, tag: component_specification_tag, result: } end |
#json_safe_and_pretty(hash_or_string) ⇒ Object
331 332 333 334 335 336 337 338 339 340 341 342 |
# File 'lib/react_on_rails/helper.rb', line 331 def json_safe_and_pretty(hash_or_string) return "{}" if hash_or_string.nil? unless hash_or_string.is_a?(String) || hash_or_string.is_a?(Hash) raise ReactOnRails::Error, "#{__method__} only accepts String or Hash as argument " \ "(#{hash_or_string.class} given)." end json_value = hash_or_string.is_a?(String) ? hash_or_string : hash_or_string.to_json ReactOnRails::JsonOutput.escape(json_value) end |
#load_pack_for_generated_component(_react_component_name, render_options) ⇒ Object
439 440 441 442 443 444 445 446 447 448 449 450 451 452 453 454 455 456 |
# File 'lib/react_on_rails/helper.rb', line 439 def load_pack_for_generated_component(_react_component_name, ) return unless .auto_load_bundle ReactOnRails::PackerUtils.raise_nested_entries_disabled unless ReactOnRails::PackerUtils.nested_entries? generated_component_name = .react_component_name if Rails.env.development? is_component_pack_present = File.exist?(generated_components_pack_path(generated_component_name)) raise_missing_autoloaded_bundle(generated_component_name) unless is_component_pack_present end = { defer: ReactOnRails.configuration.generated_component_packs_loading_strategy == :defer } # Old versions of Shakapacker don't support async script tags. # ReactOnRails.configure already validates if async loading is supported by the installed Shakapacker version. # Therefore, we only need to pass the async option if the loading strategy is explicitly set to :async [:async] = true if ReactOnRails.configuration.generated_component_packs_loading_strategy == :async append_javascript_pack_tag("generated/#{generated_component_name}", **) append_stylesheet_pack_tag("generated/#{generated_component_name}") end |
#load_pack_for_generated_store(store_name, explicit_auto_load: false) ⇒ Object
458 459 460 461 462 463 464 465 466 467 468 469 470 471 472 473 474 475 476 477 478 479 480 481 482 483 484 |
# File 'lib/react_on_rails/helper.rb', line 458 def load_pack_for_generated_store(store_name, explicit_auto_load: false) unless ReactOnRails.configuration.stores_subdirectory.present? if explicit_auto_load raise ReactOnRails::SmartError.new( error_type: :configuration_error, details: "auto_load_bundle is enabled for store " \ "'#{store_name}', but " \ "stores_subdirectory is not configured. " \ "Set config.stores_subdirectory (e.g., " \ "'ror_stores') in your ReactOnRails " \ "configuration so that store packs can " \ "be generated and loaded." ) end return end ReactOnRails::PackerUtils.raise_nested_entries_disabled unless ReactOnRails::PackerUtils.nested_entries? if Rails.env.development? is_store_pack_present = File.exist?(generated_stores_pack_path(store_name)) raise_missing_autoloaded_store_bundle(store_name) unless is_store_pack_present end = { defer: ReactOnRails.configuration.generated_component_packs_loading_strategy == :defer } [:async] = true if ReactOnRails.configuration.generated_component_packs_loading_strategy == :async append_javascript_pack_tag("generated/#{store_name}", **) end |
#merge_client_props(existing_props, client_props) ⇒ Object
861 862 863 864 865 866 867 868 |
# File 'lib/react_on_rails/helper.rb', line 861 def merge_client_props(existing_props, client_props) merged_props = existing_props.dup client_props.each do |key, value| raise_if_duplicate_client_prop_key_types!(merged_props, key) merged_props[client_prop_target_key(merged_props, key)] = value end merged_props end |
#merge_server_rendered_client_props!(render_options, result) ⇒ Object
826 827 828 829 830 831 832 833 834 835 836 837 838 839 840 841 842 843 844 845 846 847 848 849 850 851 852 853 854 855 856 857 858 859 |
# File 'lib/react_on_rails/helper.rb', line 826 def merge_server_rendered_client_props!(, result) client_props = result["clientProps"] return if client_props.nil? unless client_props.is_a?(Hash) raise ReactOnRails::Error, "Expected result[\"clientProps\"] to be a Hash, got #{client_props.class.name}." end return if client_props.empty? raw_existing_props = .props existing_props = if raw_existing_props.nil? {} elsif raw_existing_props.is_a?(String) begin JSON.parse(raw_existing_props) rescue JSON::ParserError raise ReactOnRails::Error, "Cannot merge result[\"clientProps\"] into props: failed to parse props JSON " \ "string. Ensure props is a Ruby Hash or a JSON string representing an object." end else raw_existing_props end unless existing_props.is_a?(Hash) class_name = existing_props.class.name raise ReactOnRails::Error, "Cannot merge result[\"clientProps\"] into non-Hash props. " \ "Pass props as a Hash, not #{class_name}." end .set_option(:props, merge_client_props(existing_props, client_props)) end |
#modulepreload_source?(source) ⇒ Boolean
638 639 640 641 642 643 644 645 646 647 648 649 |
# File 'lib/react_on_rails/helper.rb', line 638 def modulepreload_source?(source) # Priority order: explicit module metadata, rel/type metadata, then the .mjs extension heuristic. module_value = preload_manifest_value(source, "module") return true if module_value == true return false if module_value == false return true if preload_manifest_value(source, "rel").to_s == "modulepreload" return true if preload_manifest_value(source, "type").to_s == "module" source_path = preload_manifest_source(source).to_s.split(/[?#]/, 2).first File.extname(source_path) == ".mjs" end |
#normalize_js_stack_lines(stack) ⇒ Object
V8 stack strings begin with a "TypeError: ..." header line — and chained-exception stacks
add further non-frame headers mid-stack (e.g. Caused by: ...) — none of which match Ruby's
file:line:in 'method' backtrace format. Keep only the at <frame> lines so backtrace
cleaners and APM tools that ship the array raw don't choke on unparseable strings.
Frames are also .strip-ed to drop V8's leading indentation, which Ruby backtraces never carry.
When the stack has no at <frame> lines (e.g. a header-only or non-V8 string) this returns
[], leaving the backtrace nil rather than seeding it with unparseable header lines.
947 948 949 950 951 952 953 954 |
# File 'lib/react_on_rails/helper.rb', line 947 def normalize_js_stack_lines(stack) # Only V8 string stacks are parseable here. A non-String stack (e.g. an array of frames # from a non-V8 serializer) would otherwise be `to_s`-ed into Ruby array syntax and yield # no usable frames — guard explicitly so the no-backtrace path is intentional, not garbage. return [] unless stack.is_a?(String) stack.lines.map { |line| line.chomp.strip }.select { |line| line.start_with?("at ") } end |
#preload_crossorigin ⇒ Object
626 627 628 629 |
# File 'lib/react_on_rails/helper.rb', line 626 def preload_crossorigin cross_origin = shakapacker_integrity_config[:cross_origin] cross_origin.nil? ? "anonymous" : cross_origin end |
#preload_link_attributes(source, href:) ⇒ Object
600 601 602 603 604 605 606 607 |
# File 'lib/react_on_rails/helper.rb', line 600 def preload_link_attributes(source, href:) attributes = { href: } integrity = preload_source_integrity(source) return attributes unless integrity.present? cross_origin = shakapacker_integrity_config[:cross_origin] attributes.merge(integrity:, crossorigin: cross_origin.nil? ? preload_crossorigin : cross_origin) end |
#preload_link_for_javascript_source(source, href:) ⇒ Object
586 587 588 589 590 591 592 593 594 |
# File 'lib/react_on_rails/helper.rb', line 586 def preload_link_for_javascript_source(source, href:) attributes = preload_link_attributes(source, href:) if modulepreload_source?(source) attributes[:crossorigin] = preload_crossorigin if attributes[:crossorigin].nil? tag.link(**attributes.merge(rel: "modulepreload")) else tag.link(**attributes.merge(rel: "preload", as: "script")) end end |
#preload_link_for_source(source) ⇒ Object
575 576 577 578 579 580 581 582 583 584 |
# File 'lib/react_on_rails/helper.rb', line 575 def preload_link_for_source(source) case source.fetch(:source_type) when :javascript preload_link_for_javascript_source(source.fetch(:source), href: source.fetch(:href)) when :stylesheet preload_link_for_stylesheet_source(source.fetch(:source), href: source.fetch(:href)) else raise ArgumentError, "Unexpected preload source type: #{source.fetch(:source_type).inspect}" end end |
#preload_link_for_stylesheet_source(source, href:) ⇒ Object
596 597 598 |
# File 'lib/react_on_rails/helper.rb', line 596 def preload_link_for_stylesheet_source(source, href:) tag.link(**preload_link_attributes(source, href:).merge(rel: "preload", as: "style")) end |
#preload_manifest_key?(source, key) ⇒ Boolean
660 661 662 |
# File 'lib/react_on_rails/helper.rb', line 660 def preload_manifest_key?(source, key) source.respond_to?(:key?) && source.key?(key) end |
#preload_manifest_source(source) ⇒ Object
613 614 615 616 617 618 |
# File 'lib/react_on_rails/helper.rb', line 613 def preload_manifest_source(source) return source if source.is_a?(String) return preload_manifest_value(source, "src") if preload_manifest_key?(source, "src") raise ArgumentError, "Unexpected Shakapacker manifest source without src: #{source.inspect}" end |
#preload_manifest_value(source, key) ⇒ Object
651 652 653 654 655 656 657 658 |
# File 'lib/react_on_rails/helper.rb', line 651 def preload_manifest_value(source, key) return if source.is_a?(String) || !source.respond_to?(:[]) return source[key] if preload_manifest_key?(source, key) return source[key.to_sym] if preload_manifest_key?(source, key.to_sym) nil end |
#preload_source_integrity(source) ⇒ Object
620 621 622 623 624 |
# File 'lib/react_on_rails/helper.rb', line 620 def preload_source_integrity(source) return unless shakapacker_integrity_config[:enabled] preload_manifest_value(source, "integrity") end |
#preload_source_path(source) ⇒ Object
609 610 611 |
# File 'lib/react_on_rails/helper.rb', line 609 def preload_source_path(source) path_to_asset(preload_manifest_source(source), skip_pipeline: true) end |
#preload_sources_for_generated_pack(pack_name) ⇒ Object
534 535 536 |
# File 'lib/react_on_rails/helper.rb', line 534 def preload_sources_for_generated_pack(pack_name) preload_sources_for_javascript_pack(pack_name) + preload_sources_for_stylesheet_pack(pack_name) end |
#preload_sources_for_javascript_pack(pack_name) ⇒ Object
538 539 540 541 542 |
# File 'lib/react_on_rails/helper.rb', line 538 def preload_sources_for_javascript_pack(pack_name) preload_sources_for_pack(pack_name, type: :javascript, required: true).map do |source| { source:, source_type: :javascript } end end |
#preload_sources_for_pack(pack_name, type:, required:) ⇒ Object
550 551 552 553 554 555 556 557 558 |
# File 'lib/react_on_rails/helper.rb', line 550 def preload_sources_for_pack(pack_name, type:, required:) manifest = current_shakapacker_instance.manifest sources = if required manifest.lookup_pack_with_chunks!(pack_name, type:) else manifest.lookup_pack_with_chunks(pack_name, type:) end Array(sources).compact end |
#preload_sources_for_stylesheet_pack(pack_name) ⇒ Object
544 545 546 547 548 |
# File 'lib/react_on_rails/helper.rb', line 544 def preload_sources_for_stylesheet_pack(pack_name) preload_sources_for_pack(pack_name, type: :stylesheet, required: false).map do |source| { source:, source_type: :stylesheet } end end |
#prepend_render_rails_context(render_value) ⇒ safe_buffer
prepend the rails_context if not yet applied
791 792 793 794 795 |
# File 'lib/react_on_rails/helper.rb', line 791 def prepend_render_rails_context(render_value) rails_context_content = rails_context_if_not_already_rendered rails_context_content = rails_context_content.present? ? "#{rails_context_content}\n" : "" "#{rails_context_content}#{render_value}".html_safe end |
#props_string(props) ⇒ Object
894 895 896 |
# File 'lib/react_on_rails/helper.rb', line 894 def props_string(props) props.is_a?(String) ? props : props.to_json end |
#rails_context(server_side: true) ⇒ Object
This is the definitive list of the default values used for the rails_context, which is the
second parameter passed to both component and store Render-Functions.
This method can be called from views and from the controller, as helpers.rails_context
rubocop:disable Metrics/AbcSize, Metrics/CyclomaticComplexity
349 350 351 352 353 354 355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 378 379 380 381 382 383 384 385 386 387 388 389 390 391 392 393 394 395 396 397 398 399 400 401 402 403 404 405 406 407 408 409 410 411 412 413 |
# File 'lib/react_on_rails/helper.rb', line 349 def rails_context(server_side: true) # ALERT: Keep in sync with packages/react-on-rails/src/types/index.ts for the properties of RailsContext @rails_context ||= begin result = { componentRegistryTimeout: ReactOnRails.configuration.component_registry_timeout, railsEnv: Rails.env, inMailer: in_mailer?, # Locale settings i18nLocale: I18n.locale, i18nDefaultLocale: I18n.default_locale, rorVersion: ReactOnRails::VERSION, # TODO: v13 just use the version if existing # Pro gem availability signal (not license-valid state). rorPro: ReactOnRails::Utils.react_on_rails_pro? } if ReactOnRails::Utils.react_on_rails_pro? result[:rorProVersion] = ReactOnRails::Utils.react_on_rails_pro_version if ReactOnRails::Utils.rsc_support_enabled? rsc_payload_url = ReactOnRailsPro.configuration.rsc_payload_generation_url_path result[:rscPayloadGenerationUrlPath] = rsc_payload_url end end add_csp_nonce_to_context(result) if defined?(request) && request.present? # Check for encoding of the request's original_url and try to force-encoding the # URLs as UTF-8. This situation can occur in browsers that do not encode the # entire URL as UTF-8 already, mostly on the Windows platform (IE11 and lower). original_url_normalized = request.original_url if original_url_normalized.encoding == Encoding::BINARY original_url_normalized = original_url_normalized.force_encoding(Encoding::ISO_8859_1) .encode(Encoding::UTF_8) end # Using Addressable instead of standard URI to better deal with # non-ASCII characters (see https://github.com/shakacode/react_on_rails/pull/405) uri = Addressable::URI.parse(original_url_normalized) # uri = Addressable::URI.parse("http://foo.com:3000/posts?id=30&limit=5#time=1305298413") result.merge!( # URL settings href: uri.to_s, location: "#{uri.path}#{"?#{uri.query}" if uri.query.present?}", scheme: uri.scheme, # http host: uri.host, # foo.com port: uri.port, pathname: uri.path, # /posts search: uri.query, # id=30&limit=5 httpAcceptLanguage: request.env["HTTP_ACCEPT_LANGUAGE"] ) end if ReactOnRails.configuration.rendering_extension custom_context = ReactOnRails.configuration.rendering_extension.custom_context(self) result.merge!(custom_context) if custom_context end result end context = @rails_context.merge(serverSide: server_side) add_rsc_stream_observability_to_context(context) context end |
#rails_context_if_not_already_rendered ⇒ Object
764 765 766 767 768 769 770 771 772 773 774 775 776 777 778 |
# File 'lib/react_on_rails/helper.rb', line 764 def rails_context_if_not_already_rendered return "" if @rendered_rails_context data = rails_context(server_side: false) @rendered_rails_context = true attribution_comment = react_on_rails_attribution_comment script_tag = content_tag(:script, json_safe_and_pretty(data).html_safe, type: "application/json", id: "js-react-on-rails-context") "#{attribution_comment}\n#{script_tag}".html_safe end |
#raise_if_duplicate_client_prop_key_types!(props, key) ⇒ Object
870 871 872 873 874 875 876 877 878 |
# File 'lib/react_on_rails/helper.rb', line 870 def raise_if_duplicate_client_prop_key_types!(props, key) string_key = key.to_s symbol_key = string_key.to_sym return unless props.key?(string_key) && props.key?(symbol_key) raise ReactOnRails::Error, "Cannot merge result[\"clientProps\"] when props contains both string and symbol versions of " \ "#{string_key.inspect}. Normalize props keys before calling react_component." end |
#raise_missing_autoloaded_bundle(react_component_name) ⇒ Object
1061 1062 1063 1064 1065 1066 1067 |
# File 'lib/react_on_rails/helper.rb', line 1061 def raise_missing_autoloaded_bundle(react_component_name) raise ReactOnRails::SmartError.new( error_type: :missing_auto_loaded_bundle, component_name: react_component_name, expected_path: generated_components_pack_path(react_component_name) ) end |
#raise_missing_autoloaded_store_bundle(store_name) ⇒ Object
1069 1070 1071 1072 1073 1074 1075 |
# File 'lib/react_on_rails/helper.rb', line 1069 def raise_missing_autoloaded_store_bundle(store_name) raise ReactOnRails::SmartError.new( error_type: :missing_auto_loaded_store_bundle, component_name: store_name, expected_path: generated_stores_pack_path(store_name) ) end |
#raise_prerender_error(json_result, react_component_name, props, js_code) ⇒ Object
898 899 900 901 902 903 904 905 906 |
# File 'lib/react_on_rails/helper.rb', line 898 def raise_prerender_error(json_result, react_component_name, props, js_code) raise ReactOnRails::PrerenderError.new( component_name: react_component_name, props: sanitized_props_string(props), err: rendering_error_from_result(json_result), js_code:, console_messages: json_result["consoleReplayScript"] ) end |
#raise_renderer_prerender_error(err, react_component_name, props, js_code) ⇒ Object
908 909 910 911 912 913 914 915 916 917 918 919 920 |
# File 'lib/react_on_rails/helper.rb', line 908 def raise_renderer_prerender_error(err, react_component_name, props, js_code) prerender_error = ReactOnRails::PrerenderError.new(component_name: react_component_name, # Sanitize as this might be browser logged props: sanitized_props_string(props), err:, js_code:) renderer_parse_error = err.is_a?(ReactOnRails::JsonParseError) || (defined?(ReactOnRails::LengthPrefixedParser::ParseError) && err.is_a?(ReactOnRails::LengthPrefixedParser::ParseError)) raise prerender_error, cause: nil if renderer_parse_error raise prerender_error end |
#react_component(component_name, options = {}) ⇒ Object
Main helper methods Returns html_safe string (ActiveSupport::SafeBuffer)
101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 |
# File 'lib/react_on_rails/helper.rb', line 101 def react_component(component_name, = {}) if .key?(:immediate_hydration) ReactOnRails::Helper.warn_removed_immediate_hydration_option("react_component") .delete(:immediate_hydration) end internal_result = internal_react_component(component_name, ) server_rendered_html = internal_result[:result]["html"] console_script = internal_result[:result]["consoleReplayScript"] = internal_result[:render_options] case server_rendered_html when String html = build_react_component_result_for_server_rendered_string( server_rendered_html:, component_specification_tag: internal_result[:tag], console_script:, render_options: ) html.html_safe when Hash msg = <<~MSG Use react_component_hash (not react_component) to return a Hash to your ruby view code. See https://github.com/shakacode/react_on_rails/blob/main/react_on_rails/spec/dummy/client/app/startup/ReactHelmetApp.server.tsx for an example of the necessary javascript configuration. MSG raise ReactOnRails::Error, msg else class_name = server_rendered_html.class.name msg = <<~MSG ReactOnRails: server_rendered_html is expected to be a String or Hash for #{component_name}. Type is #{class_name} Value: #{server_rendered_html} If you're trying to use a Render-Function to return a Hash to your ruby view code, then use react_component_hash instead of react_component and see https://github.com/shakacode/react_on_rails/blob/main/react_on_rails/spec/dummy/client/app/startup/ReactHelmetApp.server.tsx for an example of the JavaScript code. MSG raise ReactOnRails::Error, msg end end |
#react_component_hash(component_name, options = {}) ⇒ Object
Returns hash with html_safe strings (ActiveSupport::SafeBuffer)
162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 |
# File 'lib/react_on_rails/helper.rb', line 162 def react_component_hash(component_name, = {}) if .key?(:immediate_hydration) ReactOnRails::Helper.warn_removed_immediate_hydration_option("react_component_hash") .delete(:immediate_hydration) end [:prerender] = true internal_result = internal_react_component(component_name, ) server_rendered_html = internal_result[:result]["html"] console_script = internal_result[:result]["consoleReplayScript"] = internal_result[:render_options] if server_rendered_html.is_a?(String) && internal_result[:result]["hasErrors"] server_rendered_html = { COMPONENT_HTML_KEY => internal_result[:result]["html"] } end if server_rendered_html.is_a?(Hash) build_react_component_result_for_server_rendered_hash( server_rendered_html:, component_specification_tag: internal_result[:tag], console_script:, render_options: ) else msg = <<~MSG Render-Function used by react_component_hash for #{component_name} is expected to return an Object. See https://github.com/shakacode/react_on_rails/blob/main/react_on_rails/spec/dummy/client/app/startup/ReactHelmetApp.server.tsx for an example of the JavaScript code. Note, your Render-Function must either take 2 params or have the property `.renderFunction = true` added to it to distinguish it from a React Function Component. MSG raise ReactOnRails::Error, msg end end |
#react_on_rails_attribution_comment ⇒ Object
Generates the HTML attribution comment Pro version calls ReactOnRailsPro::Utils for license-specific details
782 783 784 785 786 787 788 |
# File 'lib/react_on_rails/helper.rb', line 782 def react_on_rails_attribution_comment if ReactOnRails::Utils.react_on_rails_pro? ReactOnRailsPro::Utils.pro_attribution_comment else "<!-- Powered by React on Rails (c) ShakaCode | Open Source -->" end end |
#react_on_rails_preload_links(*component_names) ⇒ safe_buffer
Returns html_safe string (ActiveSupport::SafeBuffer)
36 37 38 39 40 41 42 43 44 45 46 |
# File 'sig/react_on_rails/helper.rbs', line 36 def react_on_rails_preload_links(*component_names) ReactOnRails::PackerUtils.raise_nested_entries_disabled unless ReactOnRails::PackerUtils.nested_entries? pack_names = component_names.flatten.compact.map do |component_name| generated_component_pack_name(component_name) end sources = pack_names.flat_map { |pack_name| preload_sources_for_generated_pack(pack_name) } links = unique_preload_sources_by_href(sources).map { |source| preload_link_for_source(source) } safe_join(links, "\n") end |
#redux_store(store_name, props: {}, defer: false, auto_load_bundle: nil, **rest) ⇒ Object
Returns html_safe string (ActiveSupport::SafeBuffer)
217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 |
# File 'lib/react_on_rails/helper.rb', line 217 def redux_store(store_name, props: {}, defer: false, auto_load_bundle: nil, **rest) ReactOnRails::Helper.(rest) # Auto-load store pack if configured should_auto_load = auto_load_bundle.nil? ? ReactOnRails.configuration.auto_load_bundle : auto_load_bundle load_pack_for_generated_store(store_name, explicit_auto_load: auto_load_bundle == true) if should_auto_load redux_store_data = { store_name:, props: } if defer registered_stores_defer_render << redux_store_data "YOU SHOULD NOT SEE THIS ON YOUR VIEW -- Uses as a code block, like <% redux_store %> " \ "and not <%= redux store %>" else registered_stores << redux_store_data result = render_redux_store_data(redux_store_data) prepend_render_rails_context(result).html_safe end end |
#redux_store_hydration_data ⇒ safe_buffer
Returns html_safe string (ActiveSupport::SafeBuffer)
242 243 244 245 246 247 248 |
# File 'lib/react_on_rails/helper.rb', line 242 def redux_store_hydration_data return if registered_stores_defer_render.blank? registered_stores_defer_render.reduce(+"") do |accum, redux_store_data| accum << render_redux_store_data(redux_store_data) end.html_safe end |
#registered_stores ⇒ Object
rubocop:enable Metrics/AbcSize, Metrics/CyclomaticComplexity
488 489 490 |
# File 'lib/react_on_rails/helper.rb', line 488 def registered_stores @registered_stores ||= [] end |
#registered_stores_defer_render ⇒ Object
492 493 494 |
# File 'lib/react_on_rails/helper.rb', line 492 def registered_stores_defer_render @registered_stores_defer_render ||= [] end |
#registered_stores_including_deferred ⇒ Object
496 497 498 |
# File 'lib/react_on_rails/helper.rb', line 496 def registered_stores_including_deferred registered_stores + registered_stores_defer_render end |
#render_redux_store_data(redux_store_data) ⇒ Object
888 889 890 891 892 |
# File 'lib/react_on_rails/helper.rb', line 888 def render_redux_store_data(redux_store_data) store_hydration_data = generate_store_script(redux_store_data) prepend_render_rails_context(store_hydration_data) end |
#rendering_error_from_result(json_result) ⇒ Object
922 923 924 925 926 927 928 929 930 931 932 933 934 935 936 937 |
# File 'lib/react_on_rails/helper.rb', line 922 def rendering_error_from_result(json_result) rendering_error = json_result["renderingError"] return unless rendering_error.is_a?(Hash) stack = rendering_error["stack"] # Mirror the JS `buildRSCStreamDiagnosticError` fallback: a renderingError that carries only # a stack (no message) should still surface a diagnostic rather than be silently dropped. = rendering_error["message"].presence || (stack.present? ? "RSC stream metadata reported a rendering error" : nil) return if .nil? error = StandardError.new() frames = normalize_js_stack_lines(stack) error.set_backtrace(frames) unless frames.empty? error end |
#sanitized_props_string(props) ⇒ String
262 263 264 |
# File 'lib/react_on_rails/helper.rb', line 262 def sanitized_props_string(props) ReactOnRails::JsonOutput.escape(props.is_a?(String) ? props : props.to_json) end |
#server_render_js(js_expression, options = {}) ⇒ Object
Returns html_safe string (ActiveSupport::SafeBuffer)
270 271 272 273 274 275 276 277 278 279 280 281 282 283 284 285 286 287 288 289 290 291 292 293 294 295 296 297 298 299 300 301 302 303 304 305 306 307 308 309 310 311 312 313 314 315 316 317 318 319 320 321 322 323 324 325 326 327 328 329 |
# File 'lib/react_on_rails/helper.rb', line 270 def server_render_js(js_expression, = {}) = ReactOnRails::ReactComponent::RenderOptions .new(react_component_name: "generic-js", options:) js_code = <<~JS (function() { var htmlResult = ''; var hasErrors = false; var renderingError = null; var renderingErrorObject = null; try { htmlResult = (function() { return #{js_expression}; })(); } catch(e) { renderingError = e; if (#{.throw_js_errors}) { throw e; } htmlResult = ReactOnRails.handleError({e: e, name: null, jsCode: '#{escape_javascript(js_expression)}', serverSide: true}); hasErrors = true; var errorMessage = String(renderingError); var errorStack = null; // Guard against non-Error throws (e.g., throw null / throw "string"). // Boxed primitives (for example new Boolean(false)) are objects too. if (renderingError && typeof renderingError === 'object') { if ('message' in renderingError) { errorMessage = String(renderingError.message); } // Use != (not !==) to guard both null and undefined stack values. if ('stack' in renderingError && renderingError.stack != null) { errorStack = String(renderingError.stack); } } renderingErrorObject = { message: errorMessage, stack: errorStack, }; } var consoleReplayScript = ReactOnRails.getConsoleReplayScript(); return ReactOnRails.prepareRenderResult(htmlResult, consoleReplayScript, hasErrors, renderingErrorObject); })() JS result = ReactOnRails::ServerRenderingPool .server_render_js_with_console_logging(js_code, ) html = result["html"] console_script = result["consoleReplayScript"] console_script_tag = wrap_console_script_with_nonce(console_script) if .replay_console raw("#{html}#{console_script_tag}") rescue ExecJS::ProgramError => err raise ReactOnRails::PrerenderError.new(component_name: "N/A (server_render_js called)", err:, js_code:) end |
#server_rendered_react_component(render_options) ⇒ Object
Returns object with values that are NOT html_safe!
966 967 968 969 970 971 972 973 974 975 976 977 978 979 980 981 982 983 984 985 986 987 988 989 990 991 992 993 994 995 996 997 998 999 1000 1001 1002 1003 1004 1005 1006 1007 1008 1009 1010 1011 1012 1013 1014 1015 1016 1017 1018 1019 1020 1021 1022 1023 1024 1025 1026 |
# File 'lib/react_on_rails/helper.rb', line 966 def server_rendered_react_component() return { "html" => "", "consoleReplayScript" => "" } unless .prerender react_component_name = .react_component_name props = .props # On server `location` option is added (`location = request.fullpath`) # React Router needs this to match the current route # Make sure that we use up-to-date bundle file used for server rendering, which is defined # by config file value for config.server_bundle_js_file ReactOnRails::ServerRenderingPool.reset_pool_if_server_bundle_was_modified # Since this code is not inserted on a web page, we don't need to escape props # # However, as JSON (returned from `props_string(props)`) isn't JavaScript, # but we want treat it as such, we need to compensate for the difference. # # \u2028 and \u2029 are valid characters in strings in JSON, but are treated # as newline separators in JavaScript. As no newlines are allowed in # strings in JavaScript, this causes an exception. # # We fix this by replacing these unicode characters with their escaped versions. # This should be safe, as the only place they can appear is in strings anyway. # # Read more here: http://timelessrepo.com/json-isnt-a-javascript-subset js_code = ReactOnRails::ServerRenderingJsCode.server_rendering_component_js_code( props_string: props_string(props).gsub("\u2028", '\u2028').gsub("\u2029", '\u2029'), rails_context: rails_context(server_side: true).to_json, redux_stores: initialize_redux_stores(), react_component_name:, render_options: ) begin result = ReactOnRails::ServerRenderingPool.server_render_js_with_console_logging(js_code, ) rescue StandardError => err # This error came from the renderer raise_renderer_prerender_error(err, react_component_name, props, js_code) end if .streaming? result.transform do |chunk_json_result| if should_raise_streaming_prerender_error?(chunk_json_result, ) raise_prerender_error(chunk_json_result, react_component_name, props, js_code) end # It doesn't make any transformation, it listens and raises error if a chunk has errors chunk_json_result end result.rescue do |err| # This error came from the renderer raise_renderer_prerender_error(err, react_component_name, props, js_code) end elsif result["hasErrors"] && .raise_on_prerender_error raise_prerender_error(result, react_component_name, props, js_code) end result end |
#shakapacker_integrity_config ⇒ Object
631 632 633 634 635 636 |
# File 'lib/react_on_rails/helper.rb', line 631 def shakapacker_integrity_config config = current_shakapacker_instance.config return {} unless config.respond_to?(:integrity) config.integrity || {} end |
#should_raise_streaming_prerender_error?(chunk_json_result, render_options) ⇒ Boolean
956 957 958 959 960 961 962 963 |
# File 'lib/react_on_rails/helper.rb', line 956 def should_raise_streaming_prerender_error?(chunk_json_result, ) chunk_json_result["hasErrors"] && (if chunk_json_result["isShellReady"] .raise_non_shell_server_rendering_errors else .raise_on_prerender_error end) end |
#unique_preload_sources_by_href(sources) ⇒ Object
564 565 566 567 568 569 570 571 572 573 |
# File 'lib/react_on_rails/helper.rb', line 564 def unique_preload_sources_by_href(sources) seen_hrefs = {} sources.filter_map do |source| href = preload_source_path(source.fetch(:source)) next if seen_hrefs.key?(href) seen_hrefs[href] = true source.merge(href:) end end |
#wrap_console_script_with_nonce(console_script_code) ⇒ Object
Wraps console replay JavaScript code in a script tag with CSP nonce if available. The console_script_code is already sanitized by scriptSanitizedVal() in the JavaScript layer, so using html_safe here is secure.
742 743 744 745 746 747 748 749 750 751 752 753 |
# File 'lib/react_on_rails/helper.rb', line 742 def wrap_console_script_with_nonce(console_script_code) return "" if console_script_code.blank? nonce = csp_nonce # Build the script tag with nonce if available = { id: "consoleReplayLog" } [:nonce] = nonce if nonce.present? # Safe to use html_safe because content is pre-sanitized via scriptSanitizedVal() content_tag(:script, console_script_code.html_safe, ) end |